#include "mod_commsize.h"
#include "mpi.h"

static int printed=0;

int nComm=0; // MPI_COMM_NULL should always exist as the first element
int commSizes[KEY_MAX_COMMSIZE];

int mod_commsize_init(ipm_mod_t *mod, int flags)
{
  int i;

  mod->state    = STATE_IN_INIT;
  mod->init     = mod_commsize_init;
  mod->output   = 0;
  mod->finalize = 0;
  mod->name     = "COMMSIZE";

  commSizes[0]=0; // the first element is always has size 0 and it always exist
  nComm = 1;

  mod->state    = STATE_ACTIVE;
  return IPM_OK;
}

/* range should be 64K - int is enough */
/* there is no Id 0 */
int get_commsize_id(MPI_Comm comm)
{
  int size, idx;

  if ( comm == MPI_COMM_NULL ) {
    size = 0;
  } else {
    PMPI_Comm_size(comm, &size);
  }

  for ( idx = 0; idx < nComm && idx < KEY_MAX_COMMSIZE; idx++ ) {
    if ( commSizes[idx] == size ) {
       return idx;
    }
  }

  if ( idx == KEY_MAX_COMMSIZE ) {
    // The last available index was used
    if (!printed ) {
      fprintf(stderr, "Too many comm sizes\n");
      printed = 1;
    }
    return 0;
  }
  commSizes[idx] = size;
  nComm = idx+1;

  return idx;
}
